Package: SCdeconR
Type: Package
Title: Deconvolution of Bulk RNA-Seq Data using Single-Cell RNA-Seq
        Data as Reference
Version: 1.0.2
Authors@R: person("Yuanhang", "Liu", email = "liu.yuanhang@mayo.edu", role = c("aut", "cre"))
Description: Streamlined workflow from deconvolution of bulk RNA-seq data to downstream differential expression and gene-set enrichment analysis. Provide various visualization functions. 
License: GPL (>= 3)
URL: https://github.com/Liuy12/SCdeconR/,
        https://liuy12.github.io/SCdeconR/
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: data.table, gtools, harmony, MASS, Matrix, NMF, reshape2,
        Biobase, mgcv, grDevices, glmGamPoi
Suggests: FARDEEP, reticulate, plotly, DESeq2, knitr, limma, nnls,
        preprocessCore, rmarkdown, scater, scran, SingleCellExperiment,
        doFuture, future, sctransform, Linnorm, spacexr
Depends: dplyr, edgeR, foreach, ggplot2, R (>= 2.10), Seurat
NeedsCompilation: no
Packaged: 2026-04-28 21:59:35 UTC; ext_liu_yuanhang_mayo_edu
Author: Yuanhang Liu [aut, cre]
Maintainer: Yuanhang Liu <liu.yuanhang@mayo.edu>
Repository: CRAN
Date/Publication: 2026-04-29 05:10:02 UTC
Built: R 4.6.0; ; 2026-04-29 06:26:41 UTC; unix
