# Generated by roxygen2: do not edit by hand

export(bulk_generator)
export(celltype_expression)
export(comparedeg_scatter)
export(comparegsea_scatter)
export(compute_metrics)
export(construct_ref)
export(gsea_heatmap)
export(gsea_rwplot)
export(gsea_sumplot)
export(load_scdata)
export(prepare_rnk)
export(prop_barplot)
export(read_gmt)
export(reformat_gmt)
export(run_de)
export(scaling)
export(scdecon)
export(transformation)
export(write_gmt)
import(Seurat)
import(dplyr)
import(edgeR)
import(foreach)
import(ggplot2)
importFrom(MASS,rlm)
importFrom(Matrix,colSums)
importFrom(Matrix,rowMeans)
importFrom(Matrix,rowSums)
importFrom(NMF,aheatmap)
importFrom(data.table,fread)
importFrom(data.table,fwrite)
importFrom(glmGamPoi,glm_gp)
importFrom(glmGamPoi,overdispersion_mle)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,rainbow)
importFrom(gtools,mixedsort)
importFrom(mgcv,predict.gam)
importFrom(reshape2,dcast)
importFrom(reshape2,melt)
