#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @import Seurat
#' @import dplyr
#' @import edgeR
#' @import foreach
#' @import ggplot2
#' @importFrom MASS rlm
#' @importFrom Matrix colSums
#' @importFrom Matrix rowMeans
#' @importFrom Matrix rowSums
#' @importFrom NMF aheatmap
#' @importFrom data.table fread
#' @importFrom data.table fwrite
#' @importFrom glmGamPoi glm_gp
#' @importFrom glmGamPoi overdispersion_mle
#' @importFrom grDevices colorRampPalette
#' @importFrom grDevices rainbow
#' @importFrom gtools mixedsort
#' @importFrom mgcv predict.gam
#' @importFrom reshape2 dcast
#' @importFrom reshape2 melt
## usethis namespace: end
NULL
