utils::globalVariables(c("log2foldchange.x",
                         "log2foldchange.y",
                         "category",
                         "genename",
                         "NES.x",
                         "NES.y",
                         "NAME",
                         "celltype",
                         "mixture_samep",
                         "observed_values",
                         "expected_values",
                         "mixture",
                         "i",
                         "lfcSE",
                         "padj",
                         "pvalue",
                         "baseMean",
                         "log2foldchange",
                         "log2FoldChange",
                         "pval",
                         "logCPM",
                         "PValue",
                         "pval",
                         "gsearespath",
                         "NES",
                         "SIZE",
                         "adj.P.Val",
                         "P.Value",
                         "AveExpr",
                         "padj_method",
                         "log2FC",
                         "CT",
                         "ct",
                         "ct_prop",
                         "sampleid"
                         ))
